#include "tree.h"
#include "someUtil.h"
#include "treeUtil.h"
#include "recognizeFormat.h"
#include "fastaFormat.h"
#include "talRandom.h"
#include "threeStateAlphabet.h"
#include <fstream>
#include <string>
using namespace std;


void sampleSeqs(string& inTreeFileName, string& inMSAFileName, MDOUBLE inSamplePercent, string& outTreeFileName, string& outMSAFileName);



int main(int argc, char **argv) {
	talRandom::setSeed(2);
	string inTreeFileName = argv[1];
	string inMsaDir =  argv[2];
	MDOUBLE inSamplePercent =  atof(argv[3]);
	string outDir = argv[4];
	
	int startIndex = 0;
	int endIndex = 99;
	string prefix = "LGTsim.";
	string postfix = ".aln";

	createDir("", outDir);
	for (int i = startIndex; i <= endIndex; ++i) {
		string inMSAFileName = inMsaDir + "//" + prefix + int2string(i) + postfix;
		string outTreeFileName = outDir + "//sim" + int2string(i) + ".tree";
		string outMSAFileName = outDir + "//sim" + int2string(i) + ".aln";
		sampleSeqs(inTreeFileName, inMSAFileName, inSamplePercent, outTreeFileName, outMSAFileName);
	}
	return 0;
}


void sampleSeqs(string& inTreeFileName, string& inMSAFileName, MDOUBLE inSamplePercent, string& outTreeFileName, string& outMSAFileName)
{
	tree inTree(inTreeFileName);
	threeStateAlphabet alph;
	ifstream inMSAFile(inMSAFileName.c_str());
	sequenceContainer inSc = recognizeFormat::read(inMSAFile, &alph);
	inMSAFile.close();

	int seqNumToSample = static_cast<int>(inSamplePercent * inSc.numberOfSeqs());

	///////sample sequences from MSA
	//create a new sc with the sampled sequences and a vector of names of leaves to be removed from the tree
	if (seqNumToSample > inSc.numberOfSeqs())
		errorMsg::reportError("the number of requested seqeuences is larger than the number of sequences in the MSA");
	cerr<<"eee";
	sequenceContainer newSc;
	Vint vec2Add(inSc.numberOfSeqs(),0);
	Vstring seqToRemove; 
	int n = 0;
	while (n < seqNumToSample)
	{
		int seqPlaceToAdd = talRandom::giveIntRandomNumberBetweenZeroAndEntry(inSc.numberOfSeqs());
		if (vec2Add[seqPlaceToAdd] == 0){
			vec2Add[seqPlaceToAdd] = 1;
			n++;
		}
        
	}
	for (int i = 0; i<vec2Add.size();i++){
		if (vec2Add[i] == 1)
			newSc.add(inSc[i]);
		else
			seqToRemove.push_back(inSc[i].name());
	}

	//filter tree
	for (int i=0; i<seqToRemove.size();i++){
		tree::nodeP thisNode = inTree.findNodeByName(seqToRemove[i]);
		if (thisNode) {
			inTree.removeLeaf(thisNode);
		}
		else
		{
			errorMsg::reportError("cannot find: " + seqToRemove[i]);
		}
	}

	inTree.output(outTreeFileName);
	ofstream outMSA(outMSAFileName.c_str());
	fastaFormat::write(outMSA, newSc);
	outMSA.close();
}

