#ifndef ___SIMULATE_CHANGES__
#define ___SIMULATE_CHANGES__

#include "definitions.h"
#include "tree.h"
#include "stochasticProcess.h"
#include "alphabet.h"
#include "sequenceContainer.h"
#include "chrNumModel.h"

#include <map>
#include <vector>
using namespace std;

/******************************************************************
This class simulates jumps (events) along  a 
given tree, with the aim of creating a dataset (seqeunceContainer) 
in which we know the exact number of transitions along the tree
*******************************************************************/

class simulateChangesAlongTree  {
public:
	simulateChangesAlongTree(const tree& inTree, const stochasticProcess& sp, alphabet* pAlph);
	virtual ~simulateChangesAlongTree();
	sequenceContainer simulatePosition(); 
	VVint getChangesForBranch(int nodeID);
	int getNodeContent(int nodeId) {return _nodesContent[nodeId];}
	sequenceContainer toSeqDataWithoutInternalNodes();
	int getTotalJumps(chrNumModel::jumpType type);//0=gain/1=loss/2=dupl/3=demi
	void printEvents(ostream &expFile);

private:
	void init();
	void simulateOnce(tree::nodeP curNode, MDOUBLE disFromNode, int previousContent, int whichSon = 0);
	void calcEventsTypes(); //fill _changesTypes: the number of gain/loss/dupl/demi for each node
	

private:
	tree _tree;
	stochasticProcess _sp;
	alphabet* _pAlph;

	Vdouble _waitingTimeParams;//each entry is the lambda parameter of the exponential distribution modeling the waiting time for "getting out" of state i
	//_jumpProbs[i][j] is the probability of jumping from state i to state j (given that a change has ocured).
	VVdouble _jumpProbs; 

	VVVint _changesOccurred; // number of times changes from i to j occurred , for each branch
	VVint _changesTypes; //_changesTypes[node][jumpType]; for each node, the expected number of changes for each jump type: gain, loss, dupl, demiDupl, or to maxChr.
						//lastPlace is total across tree
	Vint _nodesContent; // the actual state at each node, retrieval according to node id
	sequenceContainer _sc;
};

#endif
